const path = require('path')
const { nodeResolve } = require('@rollup/plugin-node-resolve')
const copy = require('rollup-plugin-copy')
const replace = require('@rollup/plugin-replace')
const commonjs = require('@rollup/plugin-commonjs')
const typescript = require('@rollup/plugin-typescript')
const util = require('./util')
const fs = require('fs')

const buildDir = util.buildDir
const rootDir = util.rootDir
const extensions = ['.js', '.ts']

const tsconfig = fs.existsSync(path.join(buildDir, 'tsconfig.json')) ?
  path.join(buildDir, 'tsconfig.json') :
  path.join(rootDir, 'tsconfig.json')

const defaultOpts = {
  bundleDependencies: true,
  internals: [],
  format: 'esm',
  namedExports: {},
  env: {},
}

module.exports = function (opts = {}) {
  const banner = util.banner()
  const packageJSON = util.pkg()
  const options = Object.assign({}, defaultOpts, opts)
  const peerDependencies = packageJSON.peerDependencies || {}
  const dependencies = packageJSON.dependencies || {}
  const externalDependencies = options.bundleDependencies ? Object.keys(peerDependencies) : Object.keys(Object.assign({}, dependencies, peerDependencies))
  const externals = id => externalDependencies.filter(dep => options.internals.indexOf(dep) === -1).some(dep => (new RegExp(`^${dep}`)).test(id))
  const input = path.join(buildDir, options.entry) || path.join(buildDir, 'src/index.ts')
  const outputFile = options.outputFile || path.join(buildDir, 'miniprogram_dist/index.js')
  const copyFile = [
    { src: 'src/**/*.json', dest: 'miniprogram_dist' },
    { src: 'src/**/*.wxss', dest: 'miniprogram_dist' },
    { src: 'src/**/*.wxml', dest: 'miniprogram_dist' },
  ]  
  const commonPlugins = [
    copy({
      targets: Array.isArray(options.copy) ?
        options.copy : options.copy === false ?
          [] : copyFile,
    }),
    nodeResolve({
      mainFields: ['module', 'main', 'jsnext:main', 'browser'],
      extensions,
    }),
    commonjs({
      include: /node_modules/,
    }),
    typescript({
      tslib: require('tslib'),
      typescript: require('typescript'),
      tsconfig,
    }),
  ]
  const specificConfig = {
    'esm': {
      sourceMap: false,
      external: externals,
      plugins: commonPlugins.concat([
        replace({
          preventAssignment: true,
          values: Object.assign({
            'process.env.NODE_ENV': 'process.env.NODE_ENV',
            'process.env.BUILD': JSON.stringify('production'),
          }, options.env),
        }),
      ]),
    },
  }
  const rollupConfig = specificConfig[options.format]
  const inputOptions = {
    input,
    // external: rollupConfig.external || [],
    external: [/@doraemon-ui/],
    plugins: rollupConfig.plugins || [],
    inlineDynamicImports: true,
  }
  const outputOptions = {
    file: outputFile,
    format: options.format,
    name: options.libraryName,
    sourcemap: rollupConfig.sourceMap,
    banner,
  }
  return {
    inputOptions,
    outputOptions,
  }
}
